@@ -32,7 +32,8 @@
 #include "ts/ink_platform.h"
 #include "ts/I_Layout.h"
 
-#include <string.h>
+#include <cstring>
+#include <cmath>
 #include "P_Net.h"
 #include "P_SSLConfig.h"
 #include "P_SSLUtils.h"
@@ -42,6 +43,7 @@
 
 int SSLConfig::configid                                     = 0;
 int SSLCertificateConfig::configid                          = 0;
+int SSLTicketKeyConfig::configid                            = 0;
 int SSLConfigParams::ssl_maxrecord                          = 0;
 bool SSLConfigParams::ssl_allow_client_renegotiation        = false;
 bool SSLConfigParams::ssl_ocsp_enabled                      = false;
@@ -52,30 +54,49 @@ int SSLConfigParams::ssl_handshake_timeout_in               = 0;
 size_t SSLConfigParams::session_cache_number_buckets        = 1024;
 bool SSLConfigParams::session_cache_skip_on_lock_contention = false;
 size_t SSLConfigParams::session_cache_max_bucket_size       = 100;
-init_ssl_ctx_func SSLConfigParams::init_ssl_ctx_cb          = NULL;
-load_ssl_file_func SSLConfigParams::load_ssl_file_cb        = NULL;
+init_ssl_ctx_func SSLConfigParams::init_ssl_ctx_cb          = nullptr;
+load_ssl_file_func SSLConfigParams::load_ssl_file_cb        = nullptr;
 
 // TS-3534 Wiretracing for SSL Connections
 int SSLConfigParams::ssl_wire_trace_enabled       = 0;
-char *SSLConfigParams::ssl_wire_trace_addr        = NULL;
-IpAddr *SSLConfigParams::ssl_wire_trace_ip        = NULL;
+char *SSLConfigParams::ssl_wire_trace_addr        = nullptr;
+IpAddr *SSLConfigParams::ssl_wire_trace_ip        = nullptr;
 int SSLConfigParams::ssl_wire_trace_percentage    = 0;
-char *SSLConfigParams::ssl_wire_trace_server_name = NULL;
+char *SSLConfigParams::ssl_wire_trace_server_name = nullptr;
 
 static ConfigUpdateHandler<SSLCertificateConfig> *sslCertUpdate;
 
+// Check if the ticket_key callback #define is available, and if so, enable session tickets.
+#ifdef SSL_CTX_set_tlsext_ticket_key_cb
+
+#define HAVE_OPENSSL_SESSION_TICKETS 1
+
+#endif /* SSL_CTX_set_tlsext_ticket_key_cb */
+
 SSLConfigParams::SSLConfigParams()
+{
+  ink_mutex_init(&ctxMapLock, "Context List Lock");
+  reset();
+}
+
+SSLConfigParams::~SSLConfigParams()
+{
+  cleanup();
+  ink_mutex_destroy(&ctxMapLock);
+}
+
+void
+SSLConfigParams::reset()
 {
   serverCertPathOnly = serverCertChainFilename = configFilePath = serverCACertFilename = serverCACertPath = clientCertPath =
     clientKeyPath = clientCACertFilename = clientCACertPath = cipherSuite = client_cipherSuite = dhparamsFile = serverKeyPathOnly =
-      NULL;
-
+      ticket_key_filename                                                                                     = nullptr;
+  client_ctx                                                                                                  = nullptr;
   clientCertLevel = client_verify_depth = verify_depth = clientVerify = 0;
-
-  ssl_ctx_options               = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
-  ssl_client_ctx_protocols      = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
-  ssl_session_cache             = SSL_SESSION_CACHE_MODE_SERVER_ATS_IMPL;
-  ssl_session_cache_size        = 1024 * 100;
+  ssl_ctx_options                                                     = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
+  ssl_client_ctx_protocols                                            = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
+  ssl_session_cache                                                   = SSL_SESSION_CACHE_MODE_SERVER_ATS_IMPL;
+  ssl_session_cache_size                                              = 1024 * 100;
   ssl_session_cache_num_buckets = 1024; // Sessions per bucket is ceil(ssl_session_cache_size / ssl_session_cache_num_buckets)
   ssl_session_cache_skip_on_contention = 0;
   ssl_session_cache_timeout            = 0;
@@ -83,37 +104,35 @@ SSLConfigParams::SSLConfigParams()
   configExitOnLoadError                = 0;
 }
 
-SSLConfigParams::~SSLConfigParams()
-{
-  cleanup();
-}
-
 void
 SSLConfigParams::cleanup()
 {
-  ats_free_null(serverCertChainFilename);
-  ats_free_null(serverCACertFilename);
-  ats_free_null(serverCACertPath);
-  ats_free_null(clientCertPath);
-  ats_free_null(clientKeyPath);
-  ats_free_null(clientCACertFilename);
-  ats_free_null(clientCACertPath);
-  ats_free_null(configFilePath);
-  ats_free_null(serverCertPathOnly);
-  ats_free_null(serverKeyPathOnly);
-  ats_free_null(cipherSuite);
-  ats_free_null(client_cipherSuite);
-  ats_free_null(dhparamsFile);
-  ats_free_null(ssl_wire_trace_ip);
+  serverCertChainFilename = (char *)ats_free_null(serverCertChainFilename);
+  serverCACertFilename    = (char *)ats_free_null(serverCACertFilename);
+  serverCACertPath        = (char *)ats_free_null(serverCACertPath);
+  clientCertPath          = (char *)ats_free_null(clientCertPath);
+  clientKeyPath           = (char *)ats_free_null(clientKeyPath);
+  clientCACertFilename    = (char *)ats_free_null(clientCACertFilename);
+  clientCACertPath        = (char *)ats_free_null(clientCACertPath);
+  configFilePath          = (char *)ats_free_null(configFilePath);
+  serverCertPathOnly      = (char *)ats_free_null(serverCertPathOnly);
+  serverKeyPathOnly       = (char *)ats_free_null(serverKeyPathOnly);
+  cipherSuite             = (char *)ats_free_null(cipherSuite);
+  client_cipherSuite      = (char *)ats_free_null(client_cipherSuite);
+  dhparamsFile            = (char *)ats_free_null(dhparamsFile);
+  ssl_wire_trace_ip       = (IpAddr *)ats_free_null(ssl_wire_trace_ip);
+  ticket_key_filename     = (char *)ats_free_null(ticket_key_filename);
 
-  clientCertLevel = client_verify_depth = verify_depth = clientVerify = 0;
+  freeCTXmap();
+  SSLReleaseContext(client_ctx);
+  reset();
 }
 
 /**  set_paths_helper
 
  If path is *not* absolute, consider it relative to PREFIX
  if it's empty, just take SYSCONFDIR, otherwise we can take it as-is
- if final_path is NULL, it will not be updated.
+ if final_path is nullptr, it will not be updated.
 
  XXX: Add handling for Windows?
  */
@@ -122,7 +141,7 @@ set_paths_helper(const char *path, const char *filename, char **final_path, char
 {
   if (final_path) {
     if (path && path[0] != '/') {
-      *final_path = RecConfigReadPrefixPath(NULL, path);
+      *final_path = RecConfigReadPrefixPath(nullptr, path);
     } else if (!path || path[0] == '\0') {
       *final_path = RecConfigReadConfigDir();
     } else {
@@ -131,23 +150,23 @@ set_paths_helper(const char *path, const char *filename, char **final_path, char
   }
 
   if (final_filename) {
-    *final_filename = filename ? Layout::get()->relative_to(path, filename) : NULL;
+    *final_filename = filename ? Layout::get()->relative_to(path, filename) : nullptr;
   }
 }
 
 void
 SSLConfigParams::initialize()
 {
-  char *serverCertRelativePath          = NULL;
-  char *ssl_server_private_key_path     = NULL;
-  char *CACertRelativePath              = NULL;
-  char *ssl_client_cert_filename        = NULL;
-  char *ssl_client_cert_path            = NULL;
-  char *ssl_client_private_key_filename = NULL;
-  char *ssl_client_private_key_path     = NULL;
-  char *clientCACertRelativePath        = NULL;
-  char *ssl_server_ca_cert_filename     = NULL;
-  char *ssl_client_ca_cert_filename     = NULL;
+  char *serverCertRelativePath          = nullptr;
+  char *ssl_server_private_key_path     = nullptr;
+  char *CACertRelativePath              = nullptr;
+  char *ssl_client_cert_filename        = nullptr;
+  char *ssl_client_cert_path            = nullptr;
+  char *ssl_client_private_key_filename = nullptr;
+  char *ssl_client_private_key_path     = nullptr;
+  char *clientCACertRelativePath        = nullptr;
+  char *ssl_server_ca_cert_filename     = nullptr;
+  char *ssl_client_ca_cert_filename     = nullptr;
 
   cleanup();
 
@@ -227,14 +246,14 @@ SSLConfigParams::initialize()
 
   REC_ReadConfigStringAlloc(serverCertChainFilename, "proxy.config.ssl.server.cert_chain.filename");
   REC_ReadConfigStringAlloc(serverCertRelativePath, "proxy.config.ssl.server.cert.path");
-  set_paths_helper(serverCertRelativePath, NULL, &serverCertPathOnly, NULL);
+  set_paths_helper(serverCertRelativePath, nullptr, &serverCertPathOnly, nullptr);
   ats_free(serverCertRelativePath);
 
   configFilePath = RecConfigReadConfigPath("proxy.config.ssl.server.multicert.filename");
   REC_ReadConfigInteger(configExitOnLoadError, "proxy.config.ssl.server.multicert.exit_on_load_fail");
 
   REC_ReadConfigStringAlloc(ssl_server_private_key_path, "proxy.config.ssl.server.private_key.path");
-  set_paths_helper(ssl_server_private_key_path, NULL, &serverKeyPathOnly, NULL);
+  set_paths_helper(ssl_server_private_key_path, nullptr, &serverKeyPathOnly, nullptr);
   ats_free(ssl_server_private_key_path);
 
   REC_ReadConfigStringAlloc(ssl_server_ca_cert_filename, "proxy.config.ssl.CA.cert.filename");
@@ -272,19 +291,20 @@ SSLConfigParams::initialize()
 
   // ++++++++++++++++++++++++ Client part ++++++++++++++++++++
   client_verify_depth = 7;
-  REC_ReadConfigInt32(clientVerify, "proxy.config.ssl.client.verify.server");
 
-  ssl_client_cert_filename = NULL;
-  ssl_client_cert_path     = NULL;
+  ssl_client_cert_filename = nullptr;
+  ssl_client_cert_path     = nullptr;
   REC_ReadConfigStringAlloc(ssl_client_cert_filename, "proxy.config.ssl.client.cert.filename");
   REC_ReadConfigStringAlloc(ssl_client_cert_path, "proxy.config.ssl.client.cert.path");
-  set_paths_helper(ssl_client_cert_path, ssl_client_cert_filename, NULL, &clientCertPath);
+  if (ssl_client_cert_filename && ssl_client_cert_path) {
+    set_paths_helper(ssl_client_cert_path, ssl_client_cert_filename, nullptr, &clientCertPath);
+  }
   ats_free_null(ssl_client_cert_filename);
   ats_free_null(ssl_client_cert_path);
 
   REC_ReadConfigStringAlloc(ssl_client_private_key_filename, "proxy.config.ssl.client.private_key.filename");
   REC_ReadConfigStringAlloc(ssl_client_private_key_path, "proxy.config.ssl.client.private_key.path");
-  set_paths_helper(ssl_client_private_key_path, ssl_client_private_key_filename, NULL, &clientKeyPath);
+  set_paths_helper(ssl_client_private_key_path, ssl_client_private_key_filename, nullptr, &clientKeyPath);
   ats_free_null(ssl_client_private_key_filename);
   ats_free_null(ssl_client_private_key_path);
 
@@ -305,17 +325,105 @@ SSLConfigParams::initialize()
       ssl_wire_trace_ip = new IpAddr();
       ssl_wire_trace_ip->load(ssl_wire_trace_addr);
     } else {
-      ssl_wire_trace_ip = NULL;
+      ssl_wire_trace_ip = nullptr;
     }
     // wire trace percentage of requests
     REC_EstablishStaticConfigInt32(ssl_wire_trace_percentage, "proxy.config.ssl.wire_trace_percentage");
     REC_EstablishStaticConfigStringAlloc(ssl_wire_trace_server_name, "proxy.config.ssl.wire_trace_server_name");
   } else {
-    ssl_wire_trace_addr        = NULL;
-    ssl_wire_trace_ip          = NULL;
+    ssl_wire_trace_addr        = nullptr;
+    ssl_wire_trace_ip          = nullptr;
     ssl_wire_trace_percentage  = 0;
-    ssl_wire_trace_server_name = NULL;
+    ssl_wire_trace_server_name = nullptr;
   }
+  // Enable client regardless of config file settings as remap file
+  // can cause HTTP layer to connect using SSL. But only if SSL
+  // initialization hasn't failed already.
+  client_ctx = SSLInitClientContext(this);
+  if (!client_ctx) {
+    SSLError("Can't initialize the SSL client, HTTPS in remap rules will not function");
+  } else {
+    InsertCTX(this->clientCertPath, this->client_ctx);
+  }
+}
+
+// getCTX: returns the context attached to the given certificate
+SSL_CTX *
+SSLConfigParams::getCTX(cchar *client_cert) const
+{
+  ink_mutex_acquire(&ctxMapLock);
+  auto client_ctx = ctx_map.get(client_cert);
+  ink_mutex_release(&ctxMapLock);
+  return client_ctx;
+}
+
+// InsertCTX hashes on the absolute path to the client certificate file and stores in the map
+bool
+SSLConfigParams::InsertCTX(cchar *client_cert, SSL_CTX *cctx) const
+{
+  ink_mutex_acquire(&ctxMapLock);
+  // dup is required here to avoid the nullifying of the keys stored in the map.
+  // client_cert is coming from the overridable clientcert config retrieved by the remap plugin.
+  cchar *cert = ats_strdup(client_cert);
+  // Hashmap has no delete functionality :(
+  ctx_map.put(cert, cctx);
+  ink_mutex_release(&ctxMapLock);
+  return true;
+}
+
+void
+SSLConfigParams::printCTXmap()
+{
+  Vec<cchar *> keys;
+  ctx_map.get_keys(keys);
+  for (size_t i = 0; i < keys.length(); i++)
+    Debug("ssl", "Client certificates in the map %s", keys.get(i));
+}
+void
+SSLConfigParams::freeCTXmap() const
+{
+  ink_mutex_acquire(&ctxMapLock);
+  Vec<cchar *> keys;
+  ctx_map.get_keys(keys);
+  size_t n = keys.length();
+  Debug("ssl", "freeing CTX Map");
+  for (size_t i = 0; i < n; i++) {
+    deleteKey(keys.get(i));
+    ats_free((char *)keys.get(i));
+  }
+  ctx_map.clear();
+  ink_mutex_release(&ctxMapLock);
+}
+// creates a new context attaching the provided certificate
+SSL_CTX *
+SSLConfigParams::getNewCTX(char *client_cert) const
+{
+  SSL_CTX *nclient_ctx = nullptr;
+  nclient_ctx          = SSLInitClientContext(this);
+  if (!nclient_ctx) {
+    SSLError("Can't initialize the SSL client, HTTPS in remap rules will not function");
+    return nullptr;
+  }
+  if (nclient_ctx && client_cert != nullptr) {
+    if (!SSL_CTX_use_certificate_chain_file(nclient_ctx, (const char *)client_cert)) {
+      SSLError("failed to load client certificate from %s", this->clientCertPath);
+      SSLReleaseContext(nclient_ctx);
+      return nullptr;
+    }
+  }
+  return nclient_ctx;
+}
+
+void
+SSLConfigParams::deleteKey(cchar *key) const
+{
+  SSL_CTX_free((SSL_CTX *)ctx_map.get(key));
+}
+
+SSL_CTX *
+SSLConfigParams::getClientSSL_CTX() const
+{
+  return client_ctx;
 }
 
 void
@@ -350,12 +458,10 @@ SSLCertificateConfig::startup()
 {
   sslCertUpdate = new ConfigUpdateHandler<SSLCertificateConfig>();
   sslCertUpdate->attach("proxy.config.ssl.server.multicert.filename");
-  sslCertUpdate->attach("proxy.config.ssl.server.ticket_key.filename");
   sslCertUpdate->attach("proxy.config.ssl.server.cert.path");
   sslCertUpdate->attach("proxy.config.ssl.server.private_key.path");
   sslCertUpdate->attach("proxy.config.ssl.server.cert_chain.filename");
   sslCertUpdate->attach("proxy.config.ssl.server.session_ticket.enable");
-
   // Exit if there are problems on the certificate loading and the
   // proxy.config.ssl.server.multicert.exit_on_load_fail is true
   SSLConfig::scoped_config params;
@@ -408,3 +514,67 @@ SSLCertificateConfig::release(SSLCertLookup *lookup)
 {
   configProcessor.release(configid, lookup);
 }
+
+bool
+SSLTicketParams::LoadTicket()
+{
+  cleanup();
+
+#if HAVE_OPENSSL_SESSION_TICKETS
+  ssl_ticket_key_block *keyblock = nullptr;
+
+  SSLConfig::scoped_config params;
+
+  if (REC_ReadConfigStringAlloc(ticket_key_filename, "proxy.config.ssl.server.ticket_key.filename") == REC_ERR_OKAY &&
+      ticket_key_filename != nullptr) {
+    ats_scoped_str ticket_key_path(Layout::relative_to(params->serverCertPathOnly, ticket_key_filename));
+    keyblock = ssl_create_ticket_keyblock(ticket_key_path);
+  } else {
+    keyblock = ssl_create_ticket_keyblock(nullptr);
+  }
+  if (!keyblock) {
+    Error("ticket key reloaded from %s", ticket_key_filename);
+    return false;
+  }
+  default_global_keyblock = keyblock;
+
+  Debug("ssl", "ticket key reloaded from %s", ticket_key_filename);
+  return true;
+
+#endif
+}
+
+void
+SSLTicketKeyConfig::startup()
+{
+  auto sslTicketKey = new ConfigUpdateHandler<SSLTicketKeyConfig>();
+
+  sslTicketKey->attach("proxy.config.ssl.server.ticket_key.filename");
+  SSLConfig::scoped_config params;
+  if (!reconfigure() && params->configExitOnLoadError) {
+    Fatal("Failed to load SSL ticket key file");
+  }
+}
+
+bool
+SSLTicketKeyConfig::reconfigure()
+{
+  SSLTicketParams *ticketKey = new SSLTicketParams();
+
+  if (ticketKey) {
+    if (!ticketKey->LoadTicket()) {
+      delete ticketKey;
+      return false;
+    }
+  }
+
+  configid = configProcessor.set(configid, ticketKey);
+  return true;
+}
+
+void
+SSLTicketParams::cleanup()
+{
+  ticket_block_free(default_global_keyblock);
+  ticket_key_filename = (char *)ats_free_null(ticket_key_filename);
+}
